/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright (c) 2022 HYGON Corporation . All rights reserved.
 */

#ifndef __HCT_H__
#define __HCT_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>

#define MAX_BURST_NUMS				256

/* sm2 */
#ifndef SM2_OPERAND_LEN
#define SM2_OPERAND_LEN			32
#endif
#ifndef SM2_PRIVATE_KEY_LEN
#define SM2_PRIVATE_KEY_LEN		32
#endif
#ifndef SM2_PUBLIC_KEY_LEN
#define SM2_PUBLIC_KEY_LEN		64
#endif
#ifndef SM2_VERIFY_SIZE
#define SM2_VERIFY_SIZE			64
#endif
#ifndef SM2_LP_SIZE
#define SM2_LP_SIZE			64
#endif
#ifndef SM2_KG_SIZE
#define SM2_KG_SIZE			64
#endif
#ifndef SM2_RAND_SIZE
#define SM2_RAND_SIZE			32
#endif
#ifndef SM2_DGST_LEN
#define SM2_DGST_LEN			32
#endif
#ifndef SM2_ENC_EXT_LEN
#define SM2_ENC_EXT_LEN			96
#endif

typedef struct {
	uint8_t e_key[SM2_PUBLIC_KEY_LEN];
	uint8_t d_key[SM2_PRIVATE_KEY_LEN];
	uint8_t p_data[SM2_LP_SIZE];
	uint8_t sign[SM2_VERIFY_SIZE];
	void *ctx;
} SM2_CCP_PKEY_CTX;

struct sm2_kg_burst {
	unsigned char *k;     /* in/out. 32 bytes */
	unsigned char *kg;    /* out. 64 bytes */
	uint32_t random;      /* in. 1: CCP generate random, 0: external input random*/
};

struct sm2_lp_burst {
	unsigned char *l;     /* in. 32 bytes */
	unsigned char *p;     /* in. 64 bytes */
	unsigned char *lp;    /* out. 64 bytes */
};

struct sm2_verify_burst {
	const unsigned char *dgst;	/* in.  digest, 32 bytes. */
	const unsigned char *sig;	/* in.  signature, 64 bytes. */
	const unsigned char *e_key;	/* in.  public key, 64 bytes. */
	int32_t result;			/* out. 0: success; other: fail. */
};

struct sm2_signature_burst {
	const unsigned char *dgst;	/* in.  digest, 32 bytes. */
	unsigned char *sig;		/* out. signature, 64 bytes. */
	size_t *siglen;			/* out. 64 bytes. */
	const unsigned char *d_key;	/* in.  private key, 32 bytes. */
	int32_t result;			/* out. 0: success; other: fail. */
};

struct sm2_encrypt_burst {
	const unsigned char *plain;	/* in.  plain text. */
	size_t plen;			/* in.  plain text length. */
	unsigned char *encrypt;		/* out. encrypt text. */
	size_t *elen;			/* out. encrypt text length, plen + 96 bytes. */
	const unsigned char *e_key;	/* in.  public key, 64 bytes. */
	int32_t result;			/* out. 0: success; other: fail. */
};

struct sm2_decrypt_burst {
	const unsigned char *encrypt;	/* in.  encrypt text. */
	size_t elen;			/* in.  encrypt text length. */
	unsigned char *plain;		/* out.  plain text. */
	size_t *plen;			/* out.  plain text length, elen - 96 bytes. */
	const unsigned char *d_key;	/* in.  private key, 32 bytes. */
	int32_t result;			/* out. 0: success; other: fail. */
};

int hct_ccp_sm2_akcipher_init(SM2_CCP_PKEY_CTX *sm2_ctx);

int hct_ccp_sm2_akcipher_cleanup(SM2_CCP_PKEY_CTX *sm2_ctx);

int hct_ccp_sm2_akcipher_generate_rand(SM2_CCP_PKEY_CTX *sm2_ctx,
					unsigned char *rand,
					size_t *rlen);

int hct_ccp_sm2_akcipher_get_lp(SM2_CCP_PKEY_CTX *sm2_ctx,
				const unsigned char *l, size_t llen,
				unsigned char *lp, size_t *lplen);

int hct_ccp_sm2_akcipher_get_kg(SM2_CCP_PKEY_CTX *sm2_ctx,
				const unsigned char *k, size_t klen,
				unsigned char *kg, size_t *kglen);

int hct_ccp_sm2_akcipher_verify(SM2_CCP_PKEY_CTX *sm2_ctx,
				const unsigned char *dgst, size_t dgstlen,
				const unsigned char *sig, size_t siglen);

int hct_ccp_sm2_akcipher_sign(SM2_CCP_PKEY_CTX *sm2_ctx,
				const unsigned char *dgst, size_t dgstlen,
				unsigned char *sig, size_t *siglen);

int hct_ccp_sm2_akcipher_encrypt(SM2_CCP_PKEY_CTX *sm2_ctx,
				const uint8_t *plain, size_t plen,
				uint8_t *encrypt, size_t *elen);

int hct_ccp_sm2_akcipher_decrypt(SM2_CCP_PKEY_CTX *sm2_ctx,
				uint8_t *encrypt, size_t elen,
				uint8_t *plain, size_t *plen);

int hct_ccp_sm2_akcipher_verify_burst(struct sm2_verify_burst *blk,
					uint32_t burst);

int hct_ccp_sm2_akcipher_sign_burst(struct sm2_signature_burst *blk,
					uint32_t burst);

int hct_ccp_sm2_akcipher_encrypt_burst(struct sm2_encrypt_burst *blk,
					uint32_t burst);

int hct_ccp_sm2_akcipher_decrypt_burst(struct sm2_decrypt_burst *blk,
					uint32_t burst);

int hct_ccp_sm2_akcipher_kg_burst(struct sm2_kg_burst *blk,
					uint32_t burst);

int hct_ccp_sm2_akcipher_lp_burst(struct sm2_lp_burst *blk,
					uint32_t burst);

/* sm3 */
#ifndef SM3_DIGEST_SIZE
#define SM3_DIGEST_SIZE		32
#endif
#ifndef SM3_BLOCK_SIZE
#define SM3_BLOCK_SIZE		64
#endif
#ifndef IPAD
#define IPAD			0x36
#endif
#ifndef OPAD
#define OPAD			0x5C
#endif

typedef struct {
	void *ctx;
} HCT_SM3_AUTH_CTX;

#define SM3_BURST_BUF_SIZE	128

struct sm3_digest_burst {
	const unsigned char *data;	/* in.  input text. */
	uint32_t count;			/* in.  input text length. */
	uint16_t first;			/* in.  input flag, 1: first. */
	uint16_t final;			/* in.  input flag, 1: final. */
	uintptr_t reserved1;		/* reserved. */
	unsigned long reserved2;	/* reserved. */
	unsigned char *md;		/* out. message digest, 32 bytes. */
	uint8_t buf[SM3_BURST_BUF_SIZE];/* NA.  update buffer in each round internally. */
};

struct sm3_hmac_digest_burst {
	const unsigned char *data;	/* in.  input text. */
	uint32_t count;			/* in.  input text length. */
	uint16_t first;			/* in.  input flag, 1: first. */
	uint16_t final;			/* in.  input flag, 1: final. */
	const unsigned char *key;	/* in.  hmac key. */
	unsigned long key_len;		/* in.  hmac key length, <= 64. */
	unsigned char *md;		/* out. message digest, 32 bytes. */
	uint8_t buf[SM3_BURST_BUF_SIZE];/* NA.  update buffer in each round internally. */
};

int hct_ccp_sm3_digest_init(HCT_SM3_AUTH_CTX *sm3_ctx);

int hct_ccp_sm3_digest_cleanup(HCT_SM3_AUTH_CTX *sm3_ctx);

int hct_ccp_sm3_digest_update(HCT_SM3_AUTH_CTX *sm3_ctx,
				const void *data, size_t count);

int hct_ccp_sm3_digest_final(HCT_SM3_AUTH_CTX *sm3_ctx, unsigned char *md);

int hct_ccp_sm3_digest_burst(struct sm3_digest_burst *blk,
				uint32_t burst);

int hct_ccp_sm3_hmac_digest_burst(struct sm3_hmac_digest_burst *blk,
				uint32_t burst);

int hct_ccp_sm3_digest_dupctx(HCT_SM3_AUTH_CTX *to_ctx,
				HCT_SM3_AUTH_CTX *from_ctx);

int hct_sw_sm3_digest_init(HCT_SM3_AUTH_CTX *sm3_ctx);

int hct_sw_sm3_digest_cleanup(HCT_SM3_AUTH_CTX *sm3_ctx);

int hct_sw_sm3_digest_update(HCT_SM3_AUTH_CTX *sm3_ctx,
				const void *data, size_t count);

int hct_sw_sm3_digest_final(HCT_SM3_AUTH_CTX *sm3_ctx, unsigned char *md);

int hct_sw_sm3_digest_burst(HCT_SM3_AUTH_CTX *sm3_ctx,
				struct sm3_digest_burst *blk,
				uint32_t burst);

int hct_sw_sm3_digest_dupctx(HCT_SM3_AUTH_CTX *to_ctx,
				HCT_SM3_AUTH_CTX *from_ctx);

int hct_sw_sm3_hmac_digest_init(HCT_SM3_AUTH_CTX *sm3_ctx,
				const unsigned char *key,
				size_t key_len);

int hct_sw_sm3_hmac_digest_cleanup(HCT_SM3_AUTH_CTX *sm3_ctx);

int hct_sw_sm3_hmac_digest_update(HCT_SM3_AUTH_CTX *sm3_ctx,
				const void *data, size_t count);

int hct_sw_sm3_hmac_digest_final(HCT_SM3_AUTH_CTX *sm3_ctx, unsigned char *md);

int hct_sw_sm3_hmac_digest_burst(HCT_SM3_AUTH_CTX *sm3_ctx,
				struct sm3_hmac_digest_burst *blk,
				uint32_t burst);

int hct_sw_sm3_hmac_digest_dupctx(HCT_SM3_AUTH_CTX *to_ctx,
				HCT_SM3_AUTH_CTX *from_ctx);

/* sm4 */
#ifndef SM4_BLOCK_SIZE
#define SM4_BLOCK_SIZE		16
#endif
#ifndef SM4_KEYSZ
#define SM4_KEYSZ		16
#endif
#ifndef SM4_IV_LENGTH
#define SM4_IV_LENGTH		16
#endif

enum hct_crypto_cipher_algorithm {
	HCT_CRYPTO_CIPHER_SM4_CBC,
	HCT_CRYPTO_CIPHER_SM4_CTR,
	HCT_CRYPTO_CIPHER_SM4_ECB,
	HCT_CRYPTO_CIPHER_SM4_OFB,
	HCT_CRYPTO_CIPHER_SM4_CFB,
	HCT_CRYPTO_CIPHER_SM4_MAX,
};

typedef struct {
	void *ctx;
} HCT_SM4_CIPHER_CTX;

struct sm4_cipher_burst {
	enum hct_crypto_cipher_algorithm alg;		/* in.  sm4 algorithm. */
	int enc;					/* in.  1: encrypt, 0: decrypt. */
	const uint8_t *key;				/* in.  sm4 key, 16 btyes. */
	const uint8_t *iv;				/* in/out.  sm4 initial vector, 16 bytes. */
	unsigned char *out;				/* out. sm4 output data. */
	const unsigned char *in;			/* in.  sm4 input data. */
	unsigned long inl;				/* in.  sm4 input data length. */
};

int hct_ccp_init(size_t dma_mem_size);
void hct_ccp_destroy(void);

int hct_ccp_sm4_cipher_init(HCT_SM4_CIPHER_CTX *sm4_ctx,
				const unsigned char *key,
				enum hct_crypto_cipher_algorithm alg,
				int enc);

int hct_ccp_sm4_cipher_do_cipher(HCT_SM4_CIPHER_CTX *sm4_ctx,
				unsigned char *out,
				uint8_t *iv,
				const unsigned char *in,
				size_t inl);

int hct_ccp_sm4_cipher_cleanup(HCT_SM4_CIPHER_CTX *sm4_ctx);

int hct_ccp_sm4_cipher_burst(struct sm4_cipher_burst *blk,
				uint32_t burst);

int hct_sw_sm4_cipher_init(HCT_SM4_CIPHER_CTX *sm4_ctx,
				const unsigned char *key,
				enum hct_crypto_cipher_algorithm alg,
				int enc);

int hct_sw_sm4_cipher_do_cipher(HCT_SM4_CIPHER_CTX *sm4_ctx,
				unsigned char *out,
				uint8_t *iv,
				const unsigned char *in,
				size_t inl);

int hct_sw_sm4_cipher_cleanup(HCT_SM4_CIPHER_CTX *sm4_ctx);

int hct_sw_sm4_cipher_burst(HCT_SM4_CIPHER_CTX *sm4_ctx,
				struct sm4_cipher_burst *blk,
				uint32_t burst);

/* sm4 gcm */
#define MAX_GCM_IV_LENGTH       16
#define MAX_GCM_TAG_LENGTH      16

enum GCM_CTRL_TYPE {
	GCM_CTRL_INIT,
	GCM_CTRL_AEAD_SET_IVLEN,
	GCM_CTRL_AEAD_SET_TAG,
	GCM_CTRL_AEAD_GET_TAG,
};

typedef struct {
	void *ctx;
} HCT_SM4_GCM_CIPHER_CTX;

int hct_sw_sm4_gcm_init_key(HCT_SM4_GCM_CIPHER_CTX *gctx, const uint8_t *key, const uint8_t *iv, int enc);

int hct_sw_sm4_gcm_cleanup(HCT_SM4_GCM_CIPHER_CTX *gctx);

int hct_sw_sm4_gcm_ctrl(HCT_SM4_GCM_CIPHER_CTX *gctx, int type, int arg, void *ptr);

int hct_sw_sm4_gcm_cipher(HCT_SM4_GCM_CIPHER_CTX *gctx, uint8_t *out, uint8_t *in, size_t len);

int hct_sw_sm4_gcm_cipher_dupctx(HCT_SM4_GCM_CIPHER_CTX *to_gctx, HCT_SM4_GCM_CIPHER_CTX *from_gctx);

/* crypto chain */
enum hct_chain_order {
	HCT_CHAIN_ONLY_CIPHER,
	HCT_CHAIN_ONLY_AUTH,
	HCT_CHAIN_CIPHER_BPI,
	HCT_CHAIN_CIPHER_AUTH,
	HCT_CHAIN_AUTH_CIPHER,
	HCT_CHAIN_COMBINED,
	HCT_CHAIN_NOT_SUPPORTED
};

struct hct_cipher_auth_burst {
	enum hct_chain_order chain;			/* in.  crypto chain order. */
	int rsvd;					/* reserved. */
	const unsigned char *in;			/* in.  input data. */
	unsigned long inl;				/* in.  input data length. */
	struct {
		enum hct_crypto_cipher_algorithm alg;	/* in.  sm4 algorithm. */
		int enc;				/* in.  1: encrypt, 0: decrypt. */
		const uint8_t *key;			/* in.  sm4 key, 16 btyes. */
		uint8_t *iv;				/* in/out.  sm4 initial vector, 16 bytes. */
		unsigned char *out;			/* out. sm4 output data. */
	} sm4_cipher;
	struct {
		const uint8_t *key;			/* in.  hmac key. */
		unsigned long key_len;			/* in.  hmac key legnth, <= 64. */
		unsigned char *md;			/* out. message digest, 32 bytes. */
	} sm3_hmac;
};

int hct_crypto_chain_burst(struct hct_cipher_auth_burst *mbuf, uint32_t burst);

int is_hct_verified();

int get_vq_efd(int vq_idx);
int get_vq_count(void);

#ifdef __cplusplus
}
#endif

#endif /* __HCT_H__ */
