/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright (c) 2022 HYGON Corporation . All rights reserved.
 */

#ifndef _TKM_NC_H_
#define _TKM_NC_H_

#include <stdint.h>
#include <errno.h>

#define MAX_BATCH_NUMS      128
#define CCP_MAX_BURST MAX_BATCH_NUMS

#define SM3_A 0x7380166fUL
#define SM3_B 0x4914b2b9UL
#define SM3_C 0x172442d7UL
#define SM3_D 0xda8a0600UL
#define SM3_E 0xa96f30bcUL
#define SM3_F 0x163138aaUL
#define SM3_G 0xe38dee4dUL
#define SM3_H 0xb0fb0e4eUL

#define SM2_KEY_SIZE        32
#define SM3_HASH_SIZE       32

// Log defination
#define PRT(fmt, ...) \
    printf(fmt, ##__VA_ARGS__)
#define ERR_PRT(fmt, ...) \
    printf("ERR: %s:%u: " fmt, __func__, __LINE__, ##__VA_ARGS__)
#define WARN_PRT(fmt, ...) \
    printf("WARN: %s:%u: " fmt, __func__, __LINE__, ##__VA_ARGS__)
#define INFO_PRT(fmt, ...) \
    printf("INFO: %s:%u: " fmt, __func__, __LINE__, ##__VA_ARGS__)
#define DBG_PRT(fmt, ...) \
    printf("DBG: %s:%u: " fmt, __func__, __LINE__, ##__VA_ARGS__)

#define ERR_COND(cond, err, label, fmt, ...) \
    if (cond) {                              \
        ERR_PRT(fmt, ##__VA_ARGS__);         \
        ret = err;                           \
        goto label;                          \
    }

#define ERR_COND_BATCH(cond, err, ret) \
    if (cond) {                                  \
        ret = err;                               \
        continue;                                \
    }

#define ERR_COND_FMT(cond, err, label)      \
    if (cond) {                             \
        ERR_PRT ("%s\n", strerror (errno));   \
        ret = err;                          \
        goto label;                         \
    }

enum TKM_NC_RROR_TYPE {
    TKM_NC_NORMAL = 0x0,
    TKM_NC_ERR_UNINIT,
    TKM_NC_ERR_NULL_PTR,
    TKM_NC_ERR_DEV_CREATE,
    TKM_NC_ERR_MEM,
    TKM_NC_ERR_SIZE,
    TKM_NC_ERR_CCP_VQ_STATUS,
    TKM_NC_ERR_CCP_SM2,
    TKM_NC_ERR_BATCH,
    TKM_NC_ERR_TRNG,
};

struct sm3_request {
    int32_t return_code;
    uint8_t digest[SM3_HASH_SIZE];
    uint8_t *src_data;
    uint32_t src_length;
};

struct sm3_init_request {
    int32_t return_code;
    struct sm3_ctx *ctx;
};

struct sm3_ctx {
    uint32_t V[8];
    uint64_t total_len;
    uint32_t buf_len;
    uint8_t buf[128];
};

struct sm3_update_request {
    int32_t return_code;
    struct sm3_ctx *ctx;
    uint8_t *src_data;
    uint32_t src_length;
};

struct sm3_final_request {
    int32_t return_code;
    struct sm3_ctx *ctx;
    uint8_t *src_data;
    uint32_t src_length;
    uint8_t hash[SM3_HASH_SIZE];
};

struct sm2_pubkey {
    uint8_t Qx[SM2_KEY_SIZE];
    uint8_t Qy[SM2_KEY_SIZE];
};

struct sm2_verify_request {
    int32_t return_code;
    struct sm2_pubkey pkey;
    uint8_t digest[SM3_HASH_SIZE];
    uint8_t sig[64];
};

struct sm2_keypair_gen_request {
    int32_t return_code;
    uint8_t key[SM2_KEY_SIZE];
    struct sm2_pubkey pkey;
};

struct sm2_encrypt_request {
    int32_t return_code;
    struct sm2_pubkey pkey;
    uint8_t *src_data;
    int32_t src_length;
    uint8_t *dest_data;
    int32_t dest_length;
};

/**
 * hygon ccp initialize function
 *
 * input:
 * @ NULL
 * output:
 * @ NULL
 * return:
 * @ 0: success <0: fail
 **/
int tkm_nc_init(void);

/**
 * hygon ccp cleanup function
 *
 * input:
 * @ NULL
 * output:
 * @ NULL
 * return:
 * @ 0: success <0: fail
 **/
int tkm_nc_destroy(void);

/**
 * Complete sm3 hash operations in batches
 *
 * input:
 * @ reqs: sm3 request struct that contains sm3_ctx
 * @ num: number of sm3 request
 * output:
 * @ reqs->digest: sm3 digest
 * return:
 * @ 0: success <0: fail
 **/
int ccp_sm3_batch(struct sm3_request *reqs, uint32_t num);

/**
 * Initialize sm3 ctx in batches
 *
 * input:
 * @ reqs: sm3 init struct that contains sm3_ctx
 * @ num: number of sm3 init request
 * output:
 * @ NULL
 * return:
 * @ 0: success <0: fail
 **/
int ccp_sm3_init_batch(struct sm3_init_request *reqs, uint32_t num);

/**
 * Update sm3 ctx in batches
 *
 * input:
 * @ reqs: sm3 update struct that contains sm3_ctx
 * @ num: number of sm3 update request
 * output:
 * @ NULL
 * return:
 * @ 0: success <0: fail
 **/
int ccp_sm3_update_batch(struct sm3_update_request *reqs, uint32_t num);

/**
 * Final sm3 ctx in batches
 *
 * input:
 * @ reqs: sm3 final struct that contains sm3_ctx
 * @ num: number of sm3 final request
 * output:
 * @ reqs->digest: sm3 digest
 * return:
 * @ 0: success <0: fail
 **/
int ccp_sm3_final_batch(struct sm3_final_request *reqs, uint32_t num);

/**
 * Complete sm2 verify operations in batches
 *
 * input:
 * @ reqs: sm2 verify struct that contains sm3_ctx
 * @ num: number of sm2 verify request
 * output:
 * @ reqs->return_code: sm2 verify result
 * return:
 * @ 0: success <0: fail
 **/
int ccp_sm2_verify_batch(struct sm2_verify_request *reqs, uint32_t num);

/**
 * Generate ture random unmber
 *
 * input:
 * @ buf: random unmber buffer
 * @ len: length of random unmber
 * output:
 * @ buf: random unmber
 * return:
 * @ 0: success <0: fail
 **/
int ccp_rand_get(uint8_t *buf, uint32_t len);

/**
 * Generate sm2 key pair in batch
 *
 * input:
 * @ reqs: sm2 key pair struct
 * @ num: number of key pair request
 * output:
 * @ reqs->key, reqs->sm2_pubkey: sm2 key
 * return:
 * @ 0: success <0: fail
 **/
int ccp_sm2_keypair_gen_batch(struct sm2_keypair_gen_request *reqs, uint32_t num);

/**
 * compute sm2 encrypt in batch
 *
 * input:
 * @ reqs: sm2 key pair struct
 * @ num: number of key pair request
 * output:
 * @ reqs->key, reqs->sm2_pubkey: sm2 key
 * return:
 * @ 0: success <0: fail
 **/
int ccp_sm2_encrypt_batch(struct sm2_encrypt_request *reqs, uint32_t num);

#endif /* _TKM_NC_H_ */
